@@ -5,14 +5,8 @@ Use mz tol (default 5 pmm) in XIC construction.
 XICs without neighbors within x ppm are considered specific (i.e. high selectivity). 
 Low selectivity regions will be still inspected to determine the true number of XICs.
 Leave calibration to Correspondence step.
-
-
 '''
-
-# from operator import itemgetter
-
 import numpy as np
-
 from scipy.signal import find_peaks 
 from scipy import interpolate
 from scipy.ndimage import uniform_filter1d
@@ -31,7 +25,6 @@ def sum_dict(dict1, dict2):
             new[k] = dict2[k]
     return new
 
-
 def get_thousandth_bins(mzTree, mz_tolerance_ppm=5, min_timepoints=5, min_peak_height=1000):
     '''
     Process all LC-MS spectral data into flexible bins by units of 0.001 amu.
@@ -63,7 +56,6 @@ def get_thousandth_bins(mzTree, mz_tolerance_ppm=5, min_timepoints=5, min_peak_h
             return True
 
     tol_ = 0.000001 * mz_tolerance_ppm
-
     ks = sorted([k for k,v in mzTree.items() if len(v) >= min_timepoints]) # ascending order enforced
     bins_of_bins = []
     tmp = [ks[0]]
@@ -93,11 +85,11 @@ def get_thousandth_bins(mzTree, mz_tolerance_ppm=5, min_timepoints=5, min_peak_h
 
 def extract_massTracks_(ms_expt, mz_tolerance_ppm=5, min_intensity=100, min_timepoints=5, min_peak_height=1000):
     '''
-    A mass track is an EIC for full RT range, without separating the mass traces. 
+    A mass track is an EIC for full RT range, without separating the mass traces of same m/z. 
     ms_expt = pymzml.run.Reader(f)
     return 
     rt_numbers, rt_times,
-    tracks as [( mz, rtlist, intensities ), ...]
+    tracks as [( mz, np.array(intensities at full rt range) ), ...]
     '''
     alldata = []
     rt_times = []           # in seconds
@@ -123,14 +115,14 @@ def extract_massTracks_(ms_expt, mz_tolerance_ppm=5, min_intensity=100, min_time
 
     del alldata
     rt_numbers = list(range(len(rt_times)))
+    rt_length = len(rt_numbers)
 
     good_bins = get_thousandth_bins(mzTree, mz_tolerance_ppm, min_timepoints, min_peak_height)
     tracks = []
     for bin in good_bins:
-        tracks += bin_to_mass_tracks(bin, mz_tolerance_ppm)
-    #
+        tracks += bin_to_mass_tracks(bin, rt_length, mz_tolerance_ppm)
+
     # merge tracks if m/z overlap
-    #
     tracks.sort()
     merged, to_remove = [], []
     tracks_to_merge = check_close_mzs([x[0] for x in tracks], mz_tolerance_ppm)
@@ -146,13 +138,10 @@ def extract_massTracks_(ms_expt, mz_tolerance_ppm=5, min_intensity=100, min_time
         'tracks': updated_tracks,
     }
 
-
-
 # -----------------------------------------------------------------------------
 # indexing function
 # -----------------------------------------------------------------------------
 
-
 def get_thousandth_regions(ms_expt, mz_tolerance_ppm=5, min_intensity=100, min_timepoints=5, min_peak_height=1000):
     '''
     Process all LC-MS spectral data into flexible bins by units of 0.001 amu.
@@ -234,17 +223,25 @@ def get_thousandth_regions(ms_expt, mz_tolerance_ppm=5, min_intensity=100, min_t
 # mass Tracks
 # -----------------------------------------------------------------------------
 
-def extract_single_track_(bin):
+def extract_single_track_fullrt_length(bin, rt_length):
     '''
+    New format after v1.5.
     A mass track is an EIC for full RT range, without separating the mass traces. 
     input bins in format of [(mz_int, scan_num, intensity_int), ...].
+    return a massTrack as ( mz, np.array(intensities at full rt range) ).
+    '''
+    mz = np.mean([x[0] for x in bin])
+    intensity_track = np.zeros(rt_length, dtype=np.int64)
+    for r in bin:                       # this gets max intensity on the same RT scan
+        intensity_track[r[1]] = max(r[2], intensity_track[r[1]])
+    return ( mz, intensity_track )
 
-    # To consider without zeros to optimize performance -
 
-    For peak detection, we need RT as one continuous trace (RT gaps filled by zeros).
-    Peak detection is performed at sample level when landmarks are sought, 
-    but CMAP.MassGrid has its own more careful peak deteciton.
-    
+def extract_single_track_old(bin):
+    '''
+    Phased out after v1.4.
+    A mass track is an EIC for full RT range, without separating the mass traces. 
+    input bins in format of [(mz_int, scan_num, intensity_int), ...].
     return a massTrack as ( mz, rtlist, intensities ).
     '''
     mz = np.mean([x[0] for x in bin])
@@ -259,24 +256,23 @@ def extract_single_track_(bin):
         _d[r[1]] = max(r[2], _d[r[1]])
     intensities = [_d[x] for x in rtlist]
 
-
     return ( mz, list(rtlist), intensities ) # range object is not desired - use list
 
 
-def bin_to_mass_tracks(bin_data_tuples, mz_tolerance_ppm=5):
+def bin_to_mass_tracks(bin_data_tuples, rt_length, mz_tolerance_ppm=5):
     '''
     input a flexible bin by units of 0.001 amu, in format of 
                                     [(mz, scan_num, intensity_int), ...].
     A mass track is an EIC for full RT range, without separating the mass traces. 
     An optimization step is carried out in CMAP.MassGrid, to verify m/z separation.
 
-    return massTracks as [( mz, rtlist, intensities ), ...]
+    return massTracks as ( mz, np.array(intensities at full rt range) )
     '''
     bin_data_tuples.sort()   # by m/z, ascending
     mz_range = bin_data_tuples[-1][0] - bin_data_tuples[0][0]
     mz_tolerance = bin_data_tuples[0][0] * 0.000001 * mz_tolerance_ppm   
     if mz_range < mz_tolerance:
-        return [extract_single_track_(bin_data_tuples)]
+        return [extract_single_track_fullrt_length(bin_data_tuples, rt_length)]
     else:
         num_steps = int( 5* mz_range/ mz_tolerance )     # step in 1/5 mz_tolerance
         hist, bin_edges = np.histogram([x[0] for x in bin_data_tuples], num_steps)
@@ -289,24 +285,30 @@ def bin_to_mass_tracks(bin_data_tuples, mz_tolerance_ppm=5):
             for p in peaks:
                 left = max(0, p-2)
                 right = min(p+3, num_steps)
-                tracks.append( extract_single_track_(bin_data_tuples[hist_starts[left]: hist_starts[right]]) )
+                tracks.append( extract_single_track_fullrt_length(
+                            bin_data_tuples[hist_starts[left]: hist_starts[right]], rt_length) )
             return tracks
         else:
             peak = np.argmax(hist)  # if find_peaks fails, get position of max value
             left = max(0, peak-2)
             right = min(peak+3, num_steps)
-            return [extract_single_track_(bin_data_tuples[hist_starts[left]: hist_starts[right]])]
+            return [extract_single_track_fullrt_length(bin_data_tuples[hist_starts[left]: hist_starts[right]], rt_length)]
 
 def merge_two_mass_tracks(T1, T2):
     '''
-    massTracks as [( mz, rtlist, intensities ), ...]
+    massTrack as ( mz, np.array(intensities at full rt range) )
+    '''
+    return ( 0.5 * (T1[0] + T2[0]), T1[1] + T2[1] )
+
+def merge_two_mass_tracks_old(T1, T2):
+    '''
+    massTrack as ( mz, rtlist, intensities )
     '''
     mz = 0.5 * (T1[0] + T2[0])
     d_ = sum_dict( dict(zip(T1[1], T1[2])), dict(zip(T2[1], T2[2])) )
     return (mz, list(d_.keys()), list(d_.values()))
 
 
-
 # -----------------------------------------------------------------------------
 # retention time alignment
 # -----------------------------------------------------------------------------
@@ -314,6 +316,7 @@ def merge_two_mass_tracks(T1, T2):
 def rt_lowess_calibration(good_landmark_peaks, selected_reference_landmark_peaks, sample_rt_numbers, reference_rt_numbers):
     '''
     Use LOWESS, Locally Weighted Scatterplot Smoothing, to reate correspondence between sample_rt_numbers, reference_rt_numbers.
+    Predicted numbers are skipped when outside real sample boundaries.
     checked available in statsmodels.nonparametric.smoothers_lowess, v 0.12, 0.13+
         https://www.statsmodels.org/stable/generated/statsmodels.nonparametric.smoothers_lowess.lowess.html    
         But xvals have to be forced as float array until the fix is in new release.
@@ -328,12 +331,14 @@ def rt_lowess_calibration(good_landmark_peaks, selected_reference_landmark_peaks
     Return
     ======
     rt_cal_dict, dictionary converting scan number in sample_rt_numbers to calibrated integer values.
-                    so no need to recalculate every time for each track.
+                Range matched. Only changed numbers are kept for efficiency.
     reverse_rt_cal_dict, from ref RT scan numbers to sample RT scan numbers. 
-                No  guaranty dicts are in correct range after mapping.
+                Range matched. Only changed numbers are kept for efficiency.
     '''
     # force left and right ends, to prevent runaway curve functions
-    rt_rightend_ = 1.1 * max(sample_rt_numbers)
+    reference_rt_bound = max(reference_rt_numbers)
+    sample_rt_bound = max(sample_rt_numbers)
+    rt_rightend_ = 1.1 * sample_rt_bound
     xx, yy = [0, 0, 0, ], [0, 0, 0, ]
     rt_cal = sorted([(x[0]['apex'], x[1]['apex']) for x in zip(good_landmark_peaks, selected_reference_landmark_peaks)])
 
@@ -346,14 +351,21 @@ def rt_lowess_calibration(good_landmark_peaks, selected_reference_landmark_peaks
     #
     # downgrade now for compatibility to older statsmodels
     lowess_predicted = __hacked_lowess__(yy, xx, frac= .2, it=1, xvals=sample_rt_numbers)
-
-    # Force min as 0
-    rt_cal_dict = dict(zip( sample_rt_numbers, [int(round(max(ii,0),ndigits=None)) for ii in lowess_predicted] ))
     interf = interpolate.interp1d(lowess_predicted, sample_rt_numbers, fill_value="extrapolate")
     ref_interpolated = interf( reference_rt_numbers )
-    # Force min as 0; still possible going over max rt_number but that should not cause a problem
-    reverse_rt_cal_dict = dict(zip( reference_rt_numbers, [int(round( max(ii,0) , ndigits=None)) for ii in ref_interpolated] ))
-    
+
+    lowess_predicted = [int(round(ii)) for ii in lowess_predicted]
+
+    # check inf on int(round(ii))
+
+    rt_cal_dict = dict( 
+        [(x,y) for x,y in zip(sample_rt_numbers, lowess_predicted) if x!=y and 0<=y<=reference_rt_bound] )
+
+    ref_interpolated = [int(round(ii)) for ii in ref_interpolated]
+
+    reverse_rt_cal_dict = dict(
+        [(x,y) for x,y in zip(reference_rt_numbers, ref_interpolated) if x!=y and 0<=y<=sample_rt_bound] )
+        
     return rt_cal_dict, reverse_rt_cal_dict
 
 
@@ -394,6 +406,18 @@ def dwt_rt_calibrate(good_landmark_peaks, selected_reference_landmark_peaks, sam
     pass
 
 
+def remap_intensity_track(intensity_track, new, rt_cal_dict):
+    '''
+    new = basetrack.copy(), possible longer than intensity_track
+    Remap intensity_track based on rt_cal_dict. 
+    Can we make this faster?
+    '''
+    for k,v in rt_cal_dict.items():
+        new[v] = intensity_track[k]
+    return new
+
+
+
 # -----------------------------------------------------------------------------
 # smoothing function
 # -----------------------------------------------------------------------------
