@@ -1,6 +1,5 @@
 import os
 import sys
-# import random
 import json
 import pickle
 
@@ -57,7 +56,12 @@ class ext_Experiment():
         '''
         get_reference_sample_id as sample of most number_anchor_mz_pairs
         '''
-        if self.sample_registry:
+        if self.parameters['reference']:
+            # match file name; k is sm['sample_id']
+            for k,v in self.sample_registry.items():
+                if os.path.basename(self.parameters['reference']) == os.path.basename(v['input_file']):
+                    return k
+        elif self.sample_registry:
             L = [(v['number_anchor_mz_pairs'], k) for k,v in self.sample_registry.items()]
             return sorted(L, reverse=True)[0][1]
         else:
@@ -174,33 +178,7 @@ class ext_Experiment():
             'parent_epd_id': 15}],
             'MS2_Spectra': [],
             'list_matches': [('C6H14N_100.112624', 'M[1+]', 2),
-            ('C6H13N_99.104799', 'M+H[1+]', 2)]}
-        {'interim_id': 1744,
-            'neutral_formula_mass': None,
-            'neutral_formula': None,
-            'Database_referred': [],
-            'identity': [],
-            'MS1_pseudo_Spectra': [{'id_number': 'F9723',
-                'mz': 131.11659958783318,
-                'apex': 427.0,
-                'ion_relation': 'anchor',
-                'parent_epd_id': 1744},
-            {'id_number': 'F6237',
-                'mz': 153.09836047808778,
-                'apex': 434.0,
-                'ion_relation': 'Na/H',
-                'parent_epd_id': 1744}],
-            'MS2_Spectra': []},
-        {'interim_id': 1745,
-            'neutral_formula_mass': 80.037448138,
-            'neutral_formula': 'C4H4N2',
-            'MS1_pseudo_Spectra': [{'id_number': 'F1',
-                'mz': 81.04470247776916,
-                'apex': 84.0,
-                'height': 172960.79114030077,
-                'cSelectivity': 1.0,
-                'goodness_fitting': 0.9466921897715592,
-                'snr': 35.0}]},...
+            ('C6H13N_99.104799', 'M+H[1+]', 2)]},...
         '''
         s = "[peak]id_number\tmz\trtime\tapex(scan number)\t[EmpCpd]interim_id\t[EmpCpd]ion_relation\tneutral_formula\tneutral_formula_mass\
         \tname_1st_guess\tmatched_DB_shorts\tmatched_DB_records\n"
@@ -225,6 +203,7 @@ class ext_Experiment():
 
         print("\nAnnotation of %d Empirical compounds was written to %s.\n\n" %(len(dict_empCpds), outfile))
 
+
     def export_feature_tables(self, outfile='cmap_feature_table.tsv'):
         '''
         To export two tables, one is full table under `outdir/export/`, the other filtered/preferred table under `outdir`.
@@ -257,6 +236,7 @@ class ext_Experiment():
         print("Filtered Feature table (%d x %d) was written to %s.\n" %(
                                 filtered_FeatureTable.shape[0], self.number_of_samples, outfile))
         
+        
     def export_log(self):
         '''
         Export project parameters to project.json.
