@@ -2,33 +2,42 @@
 ext_Experiment is the container for whole project data.
 Heavy lifting is in constructors.CompositeMap, 
     which contains MassGrid for correspondence, and FeatureList from feature/peak detection.
-Annotation is facilitated by jms-metabolite-services, mass2chem.
-Format of a sample: {
-            'input_file': file,
-            'ion_mode': parameters['mode'],
-            # below will be populated after processing
-            'name': '',
-            'list_scan_numbers': [],
-            'list_retention_time': [],
-            'list_mass_tracks': [], 
-            'anchor_mz_pairs': [],                    # mostly isotopic pairs to establish m/z anchors (landmarks)
-            'number_anchor_mz_pairs': -1,
-        } 
-        
+Annotation is facilitated by jms-metabolite-services, mass2chem. 
+
+
+
+        if sample_N <= 10:
+            Start by matching anchor pairs, then work thru remaining traces.
+            1. create a reference based on anchor pairs
+            2. align each sample to the reference_anchor_pairs
+        elif sample_N <= 1000:     # more samples use a different method, as peak density will be apparent in more samples.
+            single batch build
+        else:                    # even more samples should be split to batches for performance reasons
+            multiple batch build
+
+
 '''
+import time
 import random
 import multiprocessing as mp
 
 import pymzml
-from jms.io import read_table_to_peaks
-from mass2chem.epdsConstructor import epdsConstructor
 
 from .experiment import *
-from .chromatograms import extract_massTracks_        # extract_massTracks, 
+from .chromatograms import extract_massTracks_ 
 from .mass_functions import *
-from .samples import get_file_masstrack_stats
 
 # -----------------------------------------------------------------------------
+# main workflow for `process`
+# -----------------------------------------------------------------------------
+
+def read_project_dir(directory, file_pattern='.mzML'):
+    '''
+    This reads centroided LC-MS files.
+    For OpenMS based XIC workflow, file_pattern='chrom.mzML'.
+    '''
+    print("Working on ~~ %s ~~ \n\n" %directory)
+    return [os.path.join(directory, f) for f in os.listdir(directory) if file_pattern in f]
 
 def register_samples(list_input_files):
     '''
@@ -41,50 +50,54 @@ def register_samples(list_input_files):
         ii += 1
     return sample_registry
 
+def create_export_folders(parameters, time_stamp):
+    parameters['outdir'] = '_'.join([parameters['project_name'], parameters['outdir'], time_stamp]) 
+    os.mkdir(parameters['outdir'])
+    os.mkdir(os.path.join(parameters['outdir'], 'pickle'))
+    os.mkdir(os.path.join(parameters['outdir'], 'export'))
+
+
+# main workflow for `process`
 def process_project(list_input_files, parameters):
     '''
     list_input_files: Extracted ion chromatogram files.
     parameters: dictionary of most parameters.
     sample_registry include: 'status:mzml_parsing', 'status:eic', 'number_anchor_mz_pairs', 
-                'data_location', track_mzs: (mz, masstrack_id)
-
+                'data_location', track_mzs: (mz, masstrack_id), 'sample_data': {}
     '''
     sample_registry = register_samples(list_input_files)
-    shared_dict = batch_EIC_from_samples_ondisk(sample_registry, parameters)
+    if parameters['database_mode'] == 'auto':
+        if len(list_input_files) <= parameters['project_sample_number_small']:
+            parameters['database_mode'] = 'memory'
+        elif len(list_input_files) <= parameters['project_sample_number_large']:
+            parameters['database_mode'] = 'ondisk'
+        else:
+            parameters['database_mode'] = 'ondisk'       # to implement 'split' 
+
+    # time_stamp is `month daay hour minute second``
+    time_stamp = ''.join([str(x) for x in time.localtime()[1:6]])
+    if parameters['database_mode'] == 'ondisk':
+        create_export_folders(parameters, time_stamp)
+        
+    # samples are processed to mass tracks (EICs) here
+    shared_dict = batch_EIC_from_samples_(sample_registry, parameters)
     for sid, sam in sample_registry.items():
         sam['status:mzml_parsing'], sam['status:eic'], sam['number_anchor_mz_pairs'
-                ], sam['data_location'], sam['track_mzs'] = shared_dict[sid]
-
+                ], sam['data_location'], sam['track_mzs'], sam['sample_data'] = shared_dict[sid]
         sam['name'] = os.path.basename(sam['input_file']).replace('.mzML', '')
     
     # print(sample_registry)
     EE = ext_Experiment(sample_registry, parameters)
     EE.process_all()
+
+    # export is separated, as it may be limited in some environments
+    if parameters['database_mode'] != 'ondisk':
+        create_export_folders(parameters, time_stamp)
     EE.export_all()
 
+
 # -----------------------------------------------------------------------------
-# Subcommand functions
-
-def analyze_single_sample(infile, 
-            mz_tolerance_ppm=5, min_intensity=100, min_timepoints=5, min_peak_height=1000,
-            parameters={}):
-    '''
-    Analyze single mzML file and print statistics.
-    parameters are not used, just place holder to use ext_Experiment class.
-
-    to add output info on instrumentation
-
-    '''
-    print("Analysis of %s\n" %infile)
-    mz_landmarks, mode, min_peak_height_ = get_file_masstrack_stats(infile,
-                mz_tolerance_ppm, min_intensity, min_timepoints, min_peak_height)
-
-    EE = ext_Experiment({}, parameters)
-    EE.load_annotation_db()
-    mass_accuracy_ratio = EE.KCD.evaluate_mass_accuracy_ratio(mz_landmarks, mode, mz_tolerance_ppm=10)
-    # print("  Mass accuracy is estimated as %2.1f ppm." %(mass_accuracy_ratio*1000000))
-    print("\n")
-
+# estimate_min_peak_height
 
 def estimate_min_peak_height(list_input_files, 
             mz_tolerance_ppm=5, min_intensity=100, min_timepoints=5, min_peak_height=500,
@@ -117,6 +130,7 @@ def ext_estimate_min_peak_height(list_input_files,
     return dict of
     ion mode and
     an estimated parameter for min peak_height as half of the min verified landmark peaks.
+
     Extended estimate_min_peak_height for X-asari use.
     '''
     estimated, _ionmode = [], []
@@ -142,47 +156,10 @@ def ext_estimate_min_peak_height(list_input_files,
     else:
         return {'mode': _ionmode[0], 'min_peak_height': recommended}
 
-def annotate_user_featuretable(infile, parameters):
-                        # mode='pos', mz_tolerance_ppm=5):
-    '''
-    infile: tab delimited file with first row as header, first column m/z and 2nd column rtime.
-    output: two files in current directory, Feature_annotation.tsv and Annotated_empricalCompounds.json
 
-    def is_coeluted_by_distance(P1, P2, rt_tolerance=10):
-        _coeluted = False
-        if abs(P1['apex']-P2['apex']) <= rt_tolerance:
-            _coeluted = True
-        return _coeluted
-
-    '''
-    parameters['outdir'] = ''
-    mode = parameters['mode']
-    list_peaks = read_table_to_peaks(infile, 
-                                has_header=True, mz_col=0, rtime_col=1, feature_id=None ,
-                                )
-    # print("Read %d features." %len(list_peaks))
-    EE = ext_Experiment({}, parameters)
-    EE.load_annotation_db()
-    ECCON = epdsConstructor(list_peaks, mode=mode)
-    EED = ExperimentalEcpdDatabase(mode=mode)
-    EED.dict_empCpds = ECCON.peaks_to_epdDict(
-                seed_search_patterns = ECCON.seed_search_patterns, 
-                ext_search_patterns = ECCON.ext_search_patterns,
-                mz_tolerance_ppm= parameters['mz_tolerance_ppm'], 
-                coelution_function='distance',
-                check_isotope_ratio = False
-                ) 
-    EED.index_empCpds()
-    EED.extend_empCpd_annotation(EE.KCD)
-    EED.annotate_singletons(EE.KCD)
-
-    EE.export_peak_annotation(EED.dict_empCpds, EE.KCD, 'Feature_annotation')
-    # also exporting JSON
-    with open('Annotated_empricalCompounds.json', 'w', encoding='utf-8') as f:
-        json.dump(EED.dict_empCpds, f, cls=NpEncoder, ensure_ascii=False, indent=2)
 
 # -----------------------------------------------------------------------------
-# Multi-core functions
+# Mass track (EIC) extraction, multi-core parralization via multiprocessing
 
 def make_iter_parameters(sample_registry, parameters, shared_dict):
     '''
@@ -197,7 +174,7 @@ def make_iter_parameters(sample_registry, parameters, shared_dict):
     min_peak_height = parameters['min_peak_height']
     for sample in sample_registry.values():
         iters.append(
-            (sample['sample_id'], sample['input_file'], parameters['mode'],
+            (sample['sample_id'], sample['input_file'], parameters['mode'], parameters['database_mode'],
             mz_tolerance_ppm, min_intensity, min_timepoints, min_peak_height,
             os.path.join(parameters['outdir'], 'pickle', os.path.basename(sample['input_file']).replace('.mzML', '')+'.pickle'),
             shared_dict
@@ -205,16 +182,7 @@ def make_iter_parameters(sample_registry, parameters, shared_dict):
         )
     return iters
 
-
-def batch_EIC_from_samples_in_memory(sample_registry, parameters):
-    pass
-
-def batch_EIC_from_samples_to_mongo(sample_registry, parameters, cursor):
-    pass
-
-
-
-def batch_EIC_from_samples_ondisk(sample_registry, parameters):
+def batch_EIC_from_samples_(sample_registry, parameters):
     '''
     multiprocessing of mass track extraction of samples, and return shared_dict.
     More anchors mean better coverage of features, helpful to select reference sample.
@@ -224,17 +192,17 @@ def batch_EIC_from_samples_ondisk(sample_registry, parameters):
         iters = make_iter_parameters(sample_registry, parameters, shared_dict)
         # print("Number of processes ", number_processes)
         with mp.Pool( parameters['multicores'] ) as pool:
-            pool.starmap( single_sample_EICs_ondisk, iters )
+            pool.starmap( single_sample_EICs_, iters )
 
         _d = dict(shared_dict)
     return _d
 
-def single_sample_EICs_ondisk(sample_id, infile, ion_mode, 
-                    mz_tolerance_ppm, min_intensity, min_timepoints, min_peak_height, outfile, shared_dict):
-
+def single_sample_EICs_(sample_id, infile, ion_mode, database_mode,
+                    mz_tolerance_ppm, min_intensity, min_timepoints, min_peak_height, outfile, 
+                    shared_dict):
     '''
     Process infile. `shared_dict` is used to pass back information, 
-    sample_id: ('mzml_parsing', 'eic', number_anchor_mz_pairs, outfile, track_mzs)
+    sample_id: ('status:mzml_parsing', 'status:eic', number_anchor_mz_pairs, outfile, track_mzs, list_mass_tracks)
     track_mzs are used later for aligning m/z tracks.
     '''
     new = {'sample_id': sample_id, 'input_file': infile, 'ion_mode': ion_mode,}
@@ -255,23 +223,31 @@ def single_sample_EICs_ondisk(sample_id, infile, ion_mode,
             list_mass_tracks.append( {
                 'id_number': ii, 
                 'mz': track[0],
-                'rt_scan_numbers': track[1], 
-                'intensity': track[2], 
+                # 'rt_scan_numbers': track[1],              # format changed after v1.5
+                'intensity': track[1], 
                 } )
             track_mzs.append( (track[0], ii) )                  # keep a reconrd in sample registry for fast MassGrid align
             ii += 1
 
         new['list_mass_tracks'] = list_mass_tracks
         anchor_mz_pairs = find_mzdiff_pairs_from_masstracks(list_mass_tracks, mz_tolerance_ppm=mz_tolerance_ppm)
+        # find_mzdiff_pairs_from_masstracks is not too sensitive to massTrack format
         new['anchor_mz_pairs'] = anchor_mz_pairs
         new['number_anchor_mz_pairs'] = len(anchor_mz_pairs)
-        shared_dict[new['sample_id']] = ('passed', 'passed', new['number_anchor_mz_pairs'], outfile, track_mzs)
-    
-        with open(outfile, 'wb') as f:
-            pickle.dump(new, f, pickle.HIGHEST_PROTOCOL)
+
+        if database_mode == 'ondisk':
+
+            shared_dict[new['sample_id']] = ('passed', 'passed', new['number_anchor_mz_pairs'], outfile, track_mzs, {})
+            with open(outfile, 'wb') as f:
+                pickle.dump(new, f, pickle.HIGHEST_PROTOCOL)
+
+        elif database_mode == 'memory':
+            shared_dict[new['sample_id']] = ('passed', 'passed', new['number_anchor_mz_pairs'], outfile, track_mzs, 
+                                            new)
 
         print("Processed %s with %d mass tracks." %(os.path.basename(infile), ii))
 
     except:
-        shared_dict[new['sample_id']] = ('failed', '', 0, '', [])
+        # xml.etree.ElementTree.ParseError
+        shared_dict[new['sample_id']] = ('failed', '', 0, '', [], {})
         print("mzML processing error in sample %s, skipped." %infile)
